from FX import *
import Skybox


depthDivisor = 10.0
beerCoefficients = (0.1, 0.5, 0.1)
eta = 0.9
environmentMap = Skybox.cubeMap


depth = Cg("""
void main(float4 position : POSITION,

      out float4 oPosition : POSITION,
      out float3 color     : COLOR,
           
  uniform float4x4 modelViewProj,
  uniform float4x4 modelView,
  uniform float    depthDivisor)
{
  oPosition = mul(modelViewProj, position);

  float4 worldPosition = mul(modelView, position);
  color = float3(-worldPosition.z / depthDivisor, 0, 0);
}
""")

beer = Cg("""
void vertex(float4 position : POSITION,
            float4 normal   : NORMAL,

        out float4 oPosition     : POSITION,
        out float4 eyePosition   : TEXCOORD0,
        out float3 refractionVec : TEXCOORD1, 
        out float2 texCoord      : TEXCOORD2,

    uniform float4x4 modelView,
    uniform float4x4 modelViewProj,
    uniform float4x4 modelViewProjIT,
    uniform float    eta)
{

  oPosition = mul(modelViewProj, position);
  eyePosition = mul(modelView, position);

  // Divides the x and y clip coordinates with w (depth division)
  // to get normalized device coordinates. These are rescaled to
  // [0, 1] and are used as texture coordinates.
  texCoord = ((oPosition.xy / oPosition.w) + 1) / 2;
  
  // calculate refraction
  float3 eyeNormal = mul(modelViewProjIT, normal).xyz;
  eyeNormal = normalize(eyeNormal);
  float3 eyeToVert = normalize(eyePosition.xyz);
  refractionVec.xyz = refract(eyeToVert, eyeNormal, eta);
}

void fragment(float4 eyePosition   : TEXCOORD0,
              float3 refractionVec : TEXCOORD1, 
              float2 texCoord      : TEXCOORD2,
              
          out float3 color : COLOR,

      uniform float3      beerCoefficients,
      uniform float       depthDivisor,
      uniform sampler2D   depthMap,
      uniform samplerCUBE environmentMap)
{
    float3 refractColor = texCUBE(environmentMap, refractionVec).rgb; 

    float depth = tex2D(depthMap, texCoord).r * depthDivisor + eyePosition.z;
    float3 attenuation = exp(-beerCoefficients*depth);

    color = attenuation * refractColor;
}""")

depthMap = Sampler(texture = DynamicOpenGLTexture(),
                   MinFilter = GL_LINEAR,
                   MagFilter = GL_LINEAR,
                   WrapS = GL_CLAMP,
                   WrapT = GL_CLAMP)

technique = [Pass(ReadBuffer = GL_BACK,
                  CullFace = GL_FRONT,
                  VertexShader = depth()),
             Pass(CullFace = GL_BACK,
                  VertexShader = beer.vertex(),
                  FragmentShader = beer.fragment())
             ]

