from FX import *

specularColor = (1.0, 1.0, 0.0)
shininess = 80
minDepth = 2
maxDepth = 5
lightPos = (0, 4, 10, 1)

spectrumTex = Texture('spectrum.png', GL_TEXTURE_1D)

vp = Cg("""
void main(float4 position : POSITION,
          float4 normal   : NORMAL,

      out float4 oPosition   : POSITION,
      out float3 eyePosition : TEXCOORD0,
      out float3 eyeNormal   : TEXCOORD1,
    
  uniform float4x4 ModelView,
  uniform float4x4 Projection,
  uniform float4x4 ModelViewIT)
{
    float4 worldPos = mul(ModelView, position);
    
    oPosition = mul(Projection, worldPos);
    eyePosition = worldPos.xyz;
    eyeNormal = normalize(mul(ModelViewIT, normal).xyz);
}""")

fp = Cg("""
void main(float3 eyePosition : TEXCOORD0,
          float3 eyeNormal   : TEXCOORD1,

      out float3 color : COLOR,

  uniform sampler1D spectrumMap,
  uniform float3 lightPos,
  uniform float3 specularColor,
  uniform float minDepth,
  uniform float maxDepth,
  uniform float shininess)
{
   float3 lightVec = normalize(lightPos.xyz - eyePosition);
   float3 eyeVec = -normalize(eyePosition);
   float3 halfVec = (lightVec + eyeVec) / 2;
	
   float diffuse = dot(eyeNormal, lightVec);
   float specular = dot(eyeNormal, halfVec);
	
   float4 lighting = lit(diffuse, specular, shininess);
   
   float3 diffuseColor = f3tex1D(spectrumMap, 
				 (-eyePosition.z - minDepth) / 
                                 (maxDepth - minDepth));

    color = (lighting.y * diffuseColor + 
              lighting.z * specularColor);
}""")

spectrumMap = Sampler(texture = spectrumTex,
                      WrapS = GL_CLAMP)

technique = [Pass(VertexShader = vp(),
                  FragmentShader = fp())]

