import sys
sys.path.append('../pyfx')
sys.path.append('../pycg')

from FX import *
from math import *
from random import random
from Numeric import *
import PIL.Image 

furLayers = 10
furLength = 0.25
size = 32

light_position = (0.0, -10.0, 0.0, 1.0)

def frand(a,b):
    return a+random()*(b-a)

def createOffsetTextureData():
    size = 32
    global offsetTextures
    offsetTextures = []
    for layer in range(2):
        data = ones((size,size,4),Float)*0.5;
        offsetTextures.append(RawOpenGLTexture(data))

def createFurTextureData():
    for y in range(size):
        for x in range(size):
            r = frand(0.2, 0.5)
            g = frand(1.0, 1.0)
            b = frand(0.6, 0.8)
            
            if random() < 0.15:
                t = frand(0.2,0.4)
            else:
                t = frand(0.7,1.0)

            r *= t
            g *= t
            b *= t

            length = min(furLayers,int(1 + furLayers*pow(random(),3)))
            for layer in range(furLayers):
                t = max(0, 1 - 2*float(layer) / (furLayers - 1))
                textureData[layer,y,x,0] = 255*(r * (1 - 0.7 * pow(t, 1.5)))
                textureData[layer,y,x,1] = 255*(g * (1 - 0.7 * pow(t, 1.5)))
                textureData[layer,y,x,2] = 255*(b * (1 - 0.7 * pow(t, 1.5)))
                if length >= layer: 
                    t = max(0, -1 + 2 * (float)(layer+1) / length)
                    textureData[layer,y,x,3] = 255 * (1 - 0.85 * pow(t, 2))
                else:
                    textureData[layer,y,x,3] = 0

        
textureData = zeros([furLayers,size,size,4],Float)
createFurTextureData()
createOffsetTextureData()

# for layer in range(furLayers):
#     im = PIL.Image.new('L', (size,size))
#     im.fromstring(textureData[layer].tostring())
#     name = 'test-%s.png' % layer
#     im.save(name)

textures = []
for layer in range(furLayers):
    textures.append(RawOpenGLTexture(textureData[layer]))


furVertex = Cg("""
struct vfconn{
	float4 HPOS : POSITION;
	float2 bump_tex_coord : TEXCOORD0;
	float3 L : TEXCOORD1;
	float3 H : TEXCOORD2;
	float2 tex_coord : TEXCOORD3;
	float2  offset_scale : TEXCOORD4;
};

vfconn main(float4 position : POSITION,
	float4 tex_coord : TEXCOORD0,
	float4 normal : TEXCOORD1,
	float4 binormal : TEXCOORD2,
	float4 tangent : TEXCOORD3,
	float4 radius  : TEXCOORD4,
	float4 vscale : TEXCOORD5,
	
	uniform float shell_height,
	uniform float offset_scale,
	
	uniform float4 light_position,
	
	uniform float4x4 ModelViewProj,
	uniform float4x4 ModelViewI
	)
{
	vfconn OUT;

	float4 extruded_position = position + normal*shell_height;
	
	OUT.HPOS = mul(ModelViewProj, extruded_position); 
	
	OUT.tex_coord = tex_coord.xy;
	OUT.bump_tex_coord = tex_coord.xy;
	
	float3x3 object_to_tangent;
	object_to_tangent[0] = tangent.xyz;
	object_to_tangent[1] = binormal.xyz;
	object_to_tangent[2] = normal.xyz;
	
	float3 L_object_space = mul(ModelViewI, light_position) - position;
	L_object_space = normalize(L_object_space);
	float3 V_object_space = mul(ModelViewI,float4(0.0, 0.0, 0.0, 1.0)) - position;
	V_object_space = normalize(V_object_space);
	float3 H_object_space = normalize(normal.xyz + V_object_space);
	OUT.L = mul(object_to_tangent, L_object_space);
	OUT.H = mul(object_to_tangent, H_object_space);
	
	
	float2 off;
	off.x = radius.x / (radius.x + shell_height);
	off.y = radius.y / (radius.y + shell_height);
	
	OUT.offset_scale = radius.xy/(radius.xy + shell_height.xx) * vscale.xy * offset_scale;
	//OUT.offset_scale = off * vscale * offset_scale;
	
	return OUT;
}
""")

furFragment = Cg("""
struct vfconn{
	float4 HPOS : POSITION;
	float2 bump_tex_coord : TEXCOORD0;
	float3 L : TEXCOORD1;
	float3 H : TEXCOORD2;
	float2 tex_coord : TEXCOORD3;
	float2  offset_scale : TEXCOORD4;
};

float4 main(vfconn IN,
	//uniform sampler2D BumpMap,
	uniform sampler2D FurTexture) : COLOR
{
    
    //float3 N = 2*tex2D(BumpMap, IN.bump_tex_coord) - float3(1,1,1);
    //float length = sqrt(dot(N.xy, N.xy));
	
    //if (length > 1)
    //   N = N / length;
		
    //N.z = 1.0;
	
    //N = normalize(N);

    float3 N = float3(0.0, 0.0, 1.0);
    float length = 1.0;
    
    float3 L = IN.L;
    float3 H = IN.H;
    float2 tex_coord =  IN.tex_coord;
    float2 offset_scale = IN.offset_scale;

    float4 reg_1 = float4(0, 0, 0, 0);
    float4 reg_2 = float4(0, 0, 0, 0);
    reg_2.y = dot(N, H);
    reg_2.x = dot(N, L);
    reg_2.z = 0.5 + 0.5*L.z; 

    reg_1.x = dot(N.xy, N.xy) + 1;
    reg_1.xy = reg_1.x * N.xy;
    reg_1.xy = tex_coord.xy + offset_scale * reg_1.xy;

    float4 fur_color = tex2D(FurTexture, reg_1.xy);

    
    float diffuse = 0.2 + 0.2 * max(0, reg_2.z) + 0.6 * pow(1 - reg_2.x * reg_2.x, 0.5 * 6);
    float specular = pow(1 - reg_2.y*reg_2.y, 0.5 * 8);
    float shadow = pow(cos(1.67 * clamp(-0.2 -1.4*L.z,0,1)),4);
    float4 col = diffuse * fur_color + shadow * specular * fur_color.x * float4(0.5, 0.5, 0.4, 0.0);
    //col = tex2D(BumpMap, IN.bump_tex_coord);
    //col.w  = 1;
    return col;
}
""")


BumpMap = Sampler(offsetTextures[0])

def createFurPass(i):
    layer = float(i+1) / furLayers;
    length = furLength * layer;
    scale = -furLength * (1.0*layer*layer + 0.4*layer);
    
    return Pass(VertexShader = furVertex(offset_scale = scale,
                                         shell_height = length),
                FragmentShader = furFragment(FurTexture = Sampler(textures[i])),
                ZEnable = True,
                CullMode = CW,
                AlphaBlendEnable = True)
                                      

technique = [Pass()] + map(createFurPass, range(furLayers))


# physicsVertex = Cg("""
# struct vfconn{
# 	float4 HPOS : POSITION;
# 	float2 tex_coord0 : TEXCOORD0;
# 	float2 force : TEXCOORD1;
# };

# vfconn main(float4 position : POSITION,
# 	float4 tex_coord0 : TEXCOORD0,
# 	float4 normal : TEXCOORD1,
# 	float4 binormal : TEXCOORD2,
# 	float4 tangent : TEXCOORD3,
	
# 	uniform float4x4 ModelViewI,
# 	uniform float3 force,
# 	uniform float3 omega )
# {
# 	vfconn OUT;

# 	OUT.HPOS = float4(2,2,0,1)*tex_coord0 - float4(1,1,0,0);
# 	OUT.tex_coord0 = tex_coord0;
# 	force = mul((float3x3)ModelViewI,force); //transform force to model space
# 	omega = mul((float3x3)ModelViewI,omega); //transform omega to model space
# 	omega = cross(position, omega);
# 	force = force - omega;
	
# 	float fs = dot(force, binormal);
# 	float ft = dot(force, tangent);
# 	float st = dot(binormal, tangent);
# 	float det = 1.0 / (1.0 - st * st);
	
# 	OUT.force.y = det * (fs - ft * st);
# 	OUT.force.x = det * (ft - fs * st);
	
# 	return OUT;
# }
# """)

# physicsFragment = Cg("""
# struct vfconn{
# 	float4 HPOS : POSITION;
# 	float2 tex_coord0 : TEXCOORD0;
# 	float2 force : TEXCOORD1;
# };


# float4 main(vfconn IN,
# 	uniform sampler2D OffsetMap,
# 	uniform float damping) : COLOR
# {
# 	float2 offset = tex2D(OffsetMap, IN.tex_coord0).xy;
# 	float2 force = IN.force * 0.5 + 0.5;

# 	offset = lerp(force, offset, damping);
# 	float length = sqrt(dot(offset, offset));
	
# 	if (length > 1)
# 		offset = offset / length;
		
	
#     return float4(offset.xy,0,1);
# }
# """)
