
from FX import *

f0 = 0.01 
LightVec = (1,1,0,0)

shader = Cg("""
float4 phongShade(float4 normalVec, 
                  float4 lightVec,
                  int shininess,
                  float4 ambientMaterial,
                  float4 diffuseMaterial,
                  float4 specularMaterial)
{
   //standard Phong lighting
   float4 eyeVec = float4(0,0,1,0);
   float4 halfVec = normalize( lightVec + eyeVec );
   
   float4 lighting = lit(dot(normalVec,lightVec),
                         dot(normalVec,halfVec), 
                         shininess);

   return (lighting.x*ambientMaterial +
           lighting.y*diffuseMaterial +
           lighting.z*specularMaterial);
}

float computeFresnel(float4 eyeDirection, float4 normal, float f0)
{
   float cosTheta = dot( eyeDirection, normal );
   float fr = f0 + (1-f0)*(1-cosTheta);
        
   return fr;
}

void main(float4 position : POSITION,
          float4 normal   : NORMAL,

      out float4 oPosition : POSITION,
      out float4 oColor    : COLOR,

  uniform float4x4 ModelView,
  uniform float4x4 Projection,
  uniform float4x4 ModelViewIT,
  uniform float4   LightVec,
  uniform float    f0)
{
   float4 worldPos = mul( ModelView, position );
   float4 worldNormal = normalize(mul( ModelViewIT, normal ));
   float4 eyePos = float4( 0, 0, 1, 0 );
   
   float4 color = phongShade(worldNormal, LightVec, 1,
                             float4(0,0,0,1), float4(1,1,1,1), float4(1,1,1,1));
         
   float fr = computeFresnel( normalize( worldPos-eyePos ), worldNormal, f0);
   
   oPosition = mul( Projection, worldPos );
   oColor = (1-fr)*float4(1,1,1,1) + fr*color; //fr*float4(1,1,1,1); //color; 
}""")

technique = [Pass(VertexShader = shader())]

