
from FX import *

LightPos = (0,4,4,1)
AmbientMaterial = (0.1, 0.1, 0.1, 1.0)
DiffuseMaterial = (1.0, 0.0, 0.0, 1.0)
SpecularMaterial = (1.0, 1.0, 1.0, 1.0)
Shininess = 80

cg = Cg("""
void vertex(float4 position : POSITION,
            float4 normal   : NORMAL,
            
        out float4 hpos      : POSITION,
        out float3 oPosition : TEXCOORD0,
        out float3 oNormal   : TEXCOORD1,

    uniform float4x4 ModelView,
    uniform float4x4 Projection,
    uniform float4x4 ModelViewIT)
{
   float4 worldPos = mul(ModelView,position);

   hpos = mul(Projection,worldPos);
   oPosition = worldPos.xyz;
   oNormal = normalize( mul( ModelViewIT, normal ).xyz);
}

void fragment(float3 position : TEXCOORD0,
              float3 normal   : TEXCOORD1,
                    
          out float4 color : COLOR,

      uniform float3 LightPos,
      uniform float4 AmbientMaterial,
      uniform float4 DiffuseMaterial,
      uniform float4 SpecularMaterial,
      uniform float  Shininess)
{
   float3 eyeVec = float3(0,0,1);

   normal = normalize(normal);
   float3 lightVec = normalize(LightPos.xyz - position);
   float3 halfVec = normalize( lightVec + eyeVec );

   float diffuse = dot( normal, lightVec );
   float specular = dot( normal, halfVec );
   float4 lighting = lit( diffuse, specular, Shininess );
        
       
   color = AmbientMaterial + lighting.y*DiffuseMaterial + lighting.z*diffuse*SpecularMaterial;
}""")


technique = [Pass(VertexShader = cg.vertex(),
                  FragmentShader = cg.fragment())]

