
from FX import *

Kd = (0.75164, 0.60648, 0.22648, 1.0)
Ks = (0.88281, 0.755802, 0.466065, 1.0)
lightPosition = (0.0, 4.0, 4.0, 1.0)
viewerPosition = (0.0, 0.0, 1.0, 1.0)
shininess = 80.0

diffuseTex   = Texture( 'diffuseRamp.png', TEXTURE_1D )
specularTex  = Texture( 'specularRamp.png', TEXTURE_1D )
edgeTex      = Texture( 'edgeRamp.png', TEXTURE_1D )

diffuseTex.initialize()
edgeTex.initialize()
specularTex.initialize()

vs = Cg("""
void main(float4 position : POSITION,
	  float4 normal   : NORMAL, 

      out float4 oPosition    : POSITION,
      out float diffuseLight  : TEXCOORD0,
      out float specularLight : TEXCOORD1,
      out float edge          : TEXCOORD2,

  uniform float4x4 modelView,
  uniform float4x4 modelViewIT,
  uniform float4x4 projection,
  uniform float3   lightPosition,
  uniform float3   viewerPosition,
  uniform float    shininess)
{
  // Compute position
  float4 wPosition = mul(modelView, position);
  oPosition = mul(projection, wPosition);

  // Calculate diffuse lighting
  float3 N = normalize(mul(modelViewIT,normal)).xyz;
  float3 L = normalize(lightPosition - wPosition.xyz);
  diffuseLight = max(dot(N, L), 0);

  // Calculate specular lighting
  float3 V = normalize(viewerPosition - wPosition.xyz);
  float3 H = normalize(L + V);
  specularLight = pow(max(dot(N, H), 0), shininess);
  if (diffuseLight <= 0) specularLight = 0;

  // Perform edge detection
  edge = max(dot(N, V), 0); 
}""")

fs = Cg("""
void main(float diffuseLight  : TEXCOORD0,
	  float specularLight : TEXCOORD1,
	  float edge          : TEXCOORD2,

      out float4 oColor : COLOR,

  uniform float4 Kd,
  uniform float4 Ks,
  uniform sampler1D diffuseRamp,
  uniform sampler1D specularRamp,
  uniform sampler1D edgeRamp)
{
  // Apply step functions
  diffuseLight = tex1D(diffuseRamp, diffuseLight).x;
  specularLight = tex1D(specularRamp, specularLight).x;
  edge = tex1D(edgeRamp, edge).x;
	
  // Computer output color	
  oColor = edge * (Kd * diffuseLight + Ks * specularLight);
}""")


diffuseRamp  = Sampler( diffuseTex )
specularRamp = Sampler( specularTex )
edgeRamp     = Sampler( edgeTex )

technique = [Pass(ZEnable = True,
                  ZWriteEnable = True,
                  CullMode = None,
                  VertexShader = vs(),
                  FragmentShader = fs())]

