#
# Some extra texturing stuff
from OpenGL.GL import *
from OpenGL.GLU import *
from PIL import *
import PIL.Image


# Seems that PyOpenGL doesn't define these.
GL_TEXTURE_CUBE_MAP_EXT = 0x8513
GL_TEXTURE_CUBE_MAP_ARB = 0x8513
GL_NORMAL_MAP_ARB = 0x8511
GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB = 0x8515
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB = 0x8516
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB = 0x8517
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB = 0x8518
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB = 0x8519
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB = 0x851A
GL_CLAMP_TO_EDGE = 0x812F
GL_TEXTURE_WRAP_R = 0x8072

class tex_object:
	def __init__(self, tgt):
		self.target = tgt
		self.valid = 0

	def bind(self):
		if not self.valid:
			self.gen()
		glBindTexture(self.target, self.texture)

	def parameter(self, pname, i):
		if type(i) == type(int()):
			glTexParameteri(self.target, pname, i);
		elif type(i) == type(float()):
			glTexParameterf(self.target, pname, i);
		elif type(i) == type([]):
			if len(i) > 0:
				if type(i[0]) == type(int()):
					glTexParameteriv(self.target, pname, i)
				elif type(i[0]) == type(float()):
					glTexParameterfv(self.target, pname, i)

	def enable(self):
		glEnable(self.target)
	def disable(self):
		glDisable(self.target)

	def delete(self):
		if self.valid:
			glDeleteTextures(1, self.texture)
			self.valid = 0
	def is_valid(self):
		return self.valid == 1

	def gen(self):
		self.texture = glGenTextures(1)
		self.valid = 1

class tex_object_cube_map(tex_object):
	def __init__(self):
		tex_object.__init__(self, GL_TEXTURE_CUBE_MAP_ARB)

def read_tga_rgb(filename):
	
	image = Image.open(filename)
	w = image.size[0]
	h = image.size[1]
	image = image.tostring("raw", "RGB", 0, -1)

	return (image, w, h)

def make_rgb_texture(target, image, mipmap):
	if mipmap:
		gluBuild2DMipmaps(target, GL_RGB8, image[1], image[2], GL_RGB, GL_UNSIGNED_BYTE,
				  image[0])
	else:
		glTexImage2D(target, 0, GL_RGB, image[1], image[2], 0, GL_RGB, GL_UNSIGNED_BYTE,
			     image[0])

def load_tga_cubemap(string, mipmap):
	
	listing = [ [ "posx", GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB ],
		    [ "negx", GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB ],
		    [ "posy", GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB ],
		    [ "negy", GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB ],
		    [ "posz", GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB ],
		    [ "negz", GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB ]
		  ]
	for list in listing:
		filename = string % list[0]
		tgt = list[1]

		output = read_tga_rgb(filename)
		make_rgb_texture(tgt, output, mipmap)
		
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);

